@@ -19,43 +19,77 @@ use rlib '../..';
 package B::DeparseTree::P526;
 use Carp;
 
-use B qw(class main_root main_start main_cv svref_2object opnumber perlstring
-	 OPf_KIDS OPf_REF OPf_STACKED OPf_SPECIAL OPf_MOD OPf_PARENS
-	 OPf_WANT OPf_WANT_VOID OPf_WANT_SCALAR OPf_WANT_LIST
-	 OPpEXISTS_SUB OPpSORT_NUMERIC OPpSORT_INTEGER
-	 OPpLVAL_INTRO OPpOUR_INTRO OPpENTERSUB_AMPER OPpSLICE OPpCONST_BARE
-	 OPpSORT_REVERSE OPpMULTIDEREF_EXISTS OPpMULTIDEREF_DELETE
-	 OPpTARGET_MY
-	 PADNAMEt_OUTER
-	 PMf_KEEP PMf_GLOBAL PMf_CONTINUE PMf_EVAL PMf_ONCE
-	 PMf_MULTILINE PMf_SINGLELINE PMf_FOLD PMf_EXTENDED PMf_EXTENDED_MORE
-	 SVf_ROK SVpad_OUR SVf_FAKE SVs_RMG SVs_SMG
-	 SVpad_TYPED
-         CVf_METHOD
-         MDEREF_ACTION_MASK
-         MDEREF_AV_gvav_aelem
-         MDEREF_AV_gvsv_vivify_rv2av_aelem
-         MDEREF_AV_padav_aelem
-         MDEREF_AV_padsv_vivify_rv2av_aelem
-         MDEREF_AV_pop_rv2av_aelem
-         MDEREF_AV_vivify_rv2av_aelem
-         MDEREF_FLAG_last
-         MDEREF_HV_gvhv_helem
-         MDEREF_HV_gvsv_vivify_rv2hv_helem
-         MDEREF_HV_padhv_helem
-         MDEREF_HV_padsv_vivify_rv2hv_helem
-         MDEREF_HV_pop_rv2hv_helem
-         MDEREF_HV_vivify_rv2hv_helem
-         MDEREF_INDEX_MASK
-         MDEREF_INDEX_const
-         MDEREF_INDEX_gvsv
-         MDEREF_INDEX_none
-         MDEREF_INDEX_padsv
-         MDEREF_MASK
-         MDEREF_SHIFT
-         MDEREF_reload
-         OPpPADRANGE_COUNTSHIFT
-         OPpSPLIT_ASSIGN OPpSPLIT_LEX
+use B qw(
+    CVf_METHOD
+    MDEREF_ACTION_MASK
+    MDEREF_AV_gvav_aelem
+    MDEREF_AV_gvsv_vivify_rv2av_aelem
+    MDEREF_AV_padav_aelem
+    MDEREF_AV_padsv_vivify_rv2av_aelem
+    MDEREF_AV_pop_rv2av_aelem
+    MDEREF_AV_vivify_rv2av_aelem
+    MDEREF_FLAG_last
+    MDEREF_HV_gvhv_helem
+    MDEREF_HV_gvsv_vivify_rv2hv_helem
+    MDEREF_HV_padhv_helem
+    MDEREF_HV_padsv_vivify_rv2hv_helem
+    MDEREF_HV_pop_rv2hv_helem
+    MDEREF_HV_vivify_rv2hv_helem
+    MDEREF_INDEX_MASK
+    MDEREF_INDEX_const
+    MDEREF_INDEX_gvsv
+    MDEREF_INDEX_none
+    MDEREF_INDEX_padsv
+    MDEREF_MASK
+    MDEREF_SHIFT
+    MDEREF_reload
+    OPf_KIDS
+    OPf_MOD
+    OPf_PARENS
+    OPf_REF
+    OPf_SPECIAL
+    OPf_STACKED
+    OPf_WANT
+    OPf_WANT_LIST
+    OPf_WANT_SCALAR
+    OPf_WANT_VOID
+    OPpCONST_BARE
+    OPpENTERSUB_AMPER
+    OPpEXISTS_SUB
+    OPpLVAL_INTRO
+    OPpMULTIDEREF_DELETE
+    OPpMULTIDEREF_EXISTS
+    OPpOUR_INTRO
+    OPpPADRANGE_COUNTSHIFT
+    OPpSLICE
+    OPpSORT_INTEGER
+    OPpSORT_NUMERIC
+    OPpSORT_REVERSE
+    OPpSPLIT_ASSIGN OPpSPLIT_LEX
+    OPpTARGET_MY
+    PADNAMEt_OUTER
+    PMf_CONTINUE
+    PMf_EVAL
+    PMf_EXTENDED
+    PMf_EXTENDED_MORE
+    PMf_FOLD
+    PMf_GLOBAL
+    PMf_KEEP
+    PMf_MULTILINE
+    PMf_ONCE
+    PMf_SINGLELINE
+    SVf_FAKE
+    SVf_ROK SVpad_OUR
+    SVpad_TYPED
+    SVs_RMG
+    SVs_SMG
+    class
+    main_cv
+    main_root
+    main_start
+    opnumber
+    perlstring
+    svref_2object
     );
 
 use B::DeparseTree::PPfns;
@@ -90,7 +124,7 @@ require feature;
 our(@EXPORT, @ISA);
 our $VERSION = '3.2.0';
 
-@ISA = qw(Exporter B::DeparseTree::PP);
+@ISA = qw(B::DeparseTree::PP);
 
 @EXPORT = qw(slice);
 
@@ -153,67 +187,6 @@ sub pp_chdir {
     }
 }
 
-sub pp_readline {
-    my $self = shift;
-    my($op, $cx) = @_;
-    my $kid = $op->first;
-    $kid = $kid->first if $kid->name eq "rv2gv"; # <$fh>
-    if (B::Deparse::is_scalar($kid)
-	and $op->flags & OPf_SPECIAL
-	and $self->deparse($kid, 1) eq 'ARGV') {
-	my $body = [$self->deparse($kid, 1, $op)];
-	return info_from_list($op, $self, ['<', $body->[0]{text}, '>'], '',
-			      'readline_scalar', {body=>$body});
-    }
-    return $self->unop($op, $cx, "readline");
-}
-
-sub pp_rcatline {
-    my $self = shift;
-    my($op) = @_;
-    return info_from_list($op, $self, ["<", $self->gv_name($self->gv_or_padgv($op)), ">"],
-			  '', 'rcatline', {});
-}
-
-sub pp_smartmatch {
-    my ($self, $op, $cx) = @_;
-    if ($op->flags & OPf_SPECIAL) {
-	return $self->deparse($op->last, $cx, $op);
-    }
-    else {
-	binop(@_, "~~", 14);
-    }
-}
-
-sub bin_info_join($$$$$$$) {
-    my ($self, $op, $lhs, $rhs, $mid, $sep, $type) = @_;
-    my $texts = [$lhs->{text}, $mid, $rhs->{text}];
-    return info_from_list($op, $self, $texts, ' ', $type, {})
-}
-
-sub bin_info_join_maybe_parens($$$$$$$$$) {
-    my ($self, $op, $lhs, $rhs, $mid, $sep, $cx, $prec, $type) = @_;
-    my $info = bin_info_join($self, $op, $lhs, $rhs, $mid, $sep, $type);
-    $info->{text} = $self->maybe_parens($info->{text}, $cx, $prec);
-    return $info;
-}
-
-sub for_loop {
-    my $self = shift;
-    my($op, $cx, $parent) = @_;
-    my $init = $self->deparse($op, 1, $parent);
-    my $s = $op->sibling;
-    my $ll = $s->name eq "unstack" ? $s->sibling : $s->first->sibling;
-    return $self->loop_common($ll, $cx, $init);
-}
-
-sub pp_padsv {
-    my $self = shift;
-    my($op, $cx, $forbid_parens) = @_;
-    return $self->maybe_my($op, $cx, $self->padname($op->targ),
-			   $forbid_parens);
-}
-
 my @threadsv_names = B::threadsv_names;
 sub pp_threadsv {
     my $self = shift;
@@ -221,40 +194,6 @@ sub pp_threadsv {
     return $self->maybe_local_str($op, $cx, "\$" .  $threadsv_names[$op->targ]);
 }
 
-sub gv_or_padgv {
-    my $self = shift;
-    my $op = shift;
-    if (class($op) eq "PADOP") {
-	return $self->padval($op->padix);
-    } else { # class($op) eq "SVOP"
-	return $op->gv;
-    }
-}
-
-sub pp_aelemfast_lex
-{
-    my($self, $op, $cx) = @_;
-    my $name = $self->padname($op->targ);
-    $name =~ s/^@/\$/;
-    return info_from_list($op, $self, [$name, "[", ($op->private + $self->{'arybase'}), "]"],
-		      '', 'pp_aelemfast_lex', {});
-}
-
-sub pp_aelemfast
-{
-    my($self, $op, $cx) = @_;
-    # optimised PADAV, pre 5.15
-    return $self->pp_aelemfast_lex(@_) if ($op->flags & OPf_SPECIAL);
-
-    my $gv = $self->gv_or_padgv($op);
-    my($name,$quoted) = $self->stash_variable_name('@',$gv);
-    $name = $quoted ? "$name->" : '$' . $name;
-    my $i = $op->private;
-    $i -= 256 if $i > 127;
-    return info_from_list($op, $self, [$name, "[", ($op->private + $self->{'arybase'}), "]"],
-		      '', 'pp_aelemfast', {});
-}
-
 sub pp_rv2sv { maybe_local_str(@_, rv2x(@_, "\$")) }
 sub pp_rv2hv { maybe_local_str(@_, rv2x(@_, "%")) }
 sub pp_rv2gv { maybe_local_str(@_, rv2x(@_, "*")) }
@@ -410,7 +349,7 @@ sub elem
 	my $array_info = $self->deparse($array, 24, $op);
 	push @{$info->{body}}, $array_info;
 	@texts = ($array_info->{text});
-	if (is_subscriptable($array)) {
+	if (B::Deparse::is_subscriptable($array)) {
 	    push @texts, $left, $idx_info->{text}, $right;
 	    $type = 'elem_no_arrow';
 	} else {
@@ -560,139 +499,6 @@ sub pp_multideref
     return info_from_list($op, $self, \@texts, '', 'multideref', {});
 }
 
-sub pp_gelem
-{
-    my($self, $op, $cx) = @_;
-    my($glob, $part) = ($op->first, $op->last);
-    $glob = $glob->first; # skip rv2gv
-    $glob = $glob->first if $glob->name eq "rv2gv"; # this one's a bug
-    my $scope = B::Deparse::is_scope($glob);
-    $glob = $self->deparse($glob, 0);
-    $part = $self->deparse($part, 1);
-    return "*" . ($scope ? "{$glob}" : $glob) . "{$part}";
-}
-
-sub pp_lslice
-{
-    my ($self, $op, $cs) = @_;
-    my $idx = $op->first;
-    my $list = $op->last;
-    my(@elems, $kid);
-    my $list_info = $self->deparse($list, 1, $op);
-    my $idx_info = $self->deparse($idx, 1, $op);
-    return info_from_list($op, $self, ['(', $list_info->{text}, ')', '[', $idx_info->{text}, ']'],
-	'', 'lslice', {body=>[$list_info, $idx_info]});
-}
-
-sub _method
-{
-    my($self, $op, $cx) = @_;
-    my @other_ops = ($op->first);
-    my $kid = $op->first->sibling; # skip pushmark
-    my($meth, $obj, @exprs);
-    if ($kid->name eq "list" and B::Deparse::want_list $kid) {
-	# When an indirect object isn't a bareword but the args are in
-	# parens, the parens aren't part of the method syntax (the LLAFR
-	# doesn't apply), but they make a list with OPf_PARENS set that
-	# doesn't get flattened by the append_elem that adds the method,
-	# making a (object, arg1, arg2, ...) list where the object
-	# usually is. This can be distinguished from
-	# '($obj, $arg1, $arg2)->meth()' (which is legal if $arg2 is an
-	# object) because in the later the list is in scalar context
-	# as the left side of -> always is, while in the former
-	# the list is in list context as method arguments always are.
-	# (Good thing there aren't method prototypes!)
-	$meth = $kid->sibling;
-	push  @other_ops, $kid->first;
-	$kid = $kid->first->sibling; # skip pushmark
-	$obj = $kid;
-	$kid = $kid->sibling;
-	for (; not B::Deparse::null $kid; $kid = $kid->sibling) {
-	    push @exprs, $kid;
-	}
-    } else {
-	$obj = $kid;
-	$kid = $kid->sibling;
-	for (; !B::Deparse::null ($kid->sibling) && $kid->name!~/^method(?:_named)?\z/;
-	     $kid = $kid->sibling) {
-	    push @exprs, $kid
-	}
-	$meth = $kid;
-    }
-
-    if ($meth->name eq "method_named") {
-	$meth = $self->meth_sv($meth)->PV;
-    } elsif ($meth->name eq "method_super") {
-	$meth = "SUPER::".$self->meth_sv($meth)->PV;
-    } elsif ($meth->name eq "method_redir") {
-        $meth = $self->meth_rclass_sv($meth)->PV.'::'.$self->meth_sv($meth)->PV;
-    } elsif ($meth->name eq "method_redir_super") {
-        $meth = $self->meth_rclass_sv($meth)->PV.'::SUPER::'.
-                $self->meth_sv($meth)->PV;
-    } else {
-	$meth = $meth->first;
-	if ($meth->name eq "const") {
-	    # As of 5.005_58, this case is probably obsoleted by the
-	    # method_named case above
-	    $meth = $self->const_sv($meth)->PV; # needs to be bare
-	}
-    }
-
-    return {
-	method => $meth,
-	variable_method => ref($meth),
-	object => $obj,
-	args => \@exprs,
-	other_ops => \@other_ops
-    }, $cx;
-}
-
-sub e_method {
-    my ($self, $op, $minfo, $cx) = @_;
-    my $obj = $self->deparse($minfo->{object}, 24, $op);
-    my @body = ($obj);
-    my $other_ops = $minfo->{other_ops};
-
-    my $meth = $minfo->{method};
-    my $meth_info;
-    if ($minfo->{variable_method}) {
-	$meth_info = $self->deparse($meth, 1, $op);
-	push @body, $meth_info;
-    }
-    my @args = map { $self->deparse($_, 6, $op) } @{$minfo->{args}};
-    push @body, @args;
-    my @args_texts = map $_->{text}, @args;
-    my $args = join(", ", @args_texts);
-
-    my $opts = {body => \@body, other_ops => $other_ops};
-    my @texts = ();
-    my $type;
-
-    if ($minfo->{object}->name eq 'scope' && B::Deparse::want_list $minfo->{object}) {
-	# method { $object }
-	# This must be deparsed this way to preserve list context
-	# of $object.
-	my $need_paren = $cx >= 6;
-	if ($need_paren) {
-	    @texts = ('(', $meth,  substr($obj,2),
-		      $args, ')');
-	    $type = 'e_method_list_paren';
-	} else {
-	    @texts = ($meth,  substr($obj,2), $args);
-	    $type = 'e_method_list';
-	}
-	return info_from_list($op, $self, \@texts, '', $type, $opts);
-    }
-    if (length $args) {
-	@texts = ($obj->{text}, '->', $meth, '(', $args, ')');
-	$type = 'e_method_args';
-    } else {
-	@texts = ($obj->{text}, '->', $meth);
-	$type = 'e_method_null';
-    }
-    return info_from_list($op, $self, \@texts, '', $type, $opts);
-}
-
 # returns "&"  and the argument bodies if the prototype doesn't match the args,
 # or ("", $args_after_prototype_demunging) if it does.
 sub check_proto {
@@ -774,53 +580,6 @@ sub check_proto {
     return ('', \@reals);
 }
 
-sub pp_enterwrite { unop(@_, "write") }
-
-# Split a floating point number into an integer mantissa and a binary
-# exponent. Assumes you've already made sure the number isn't zero or
-# some weird infinity or NaN.
-sub split_float {
-    my($f) = @_;
-    my $exponent = 0;
-    if ($f == int($f)) {
-	while ($f % 2 == 0) {
-	    $f /= 2;
-	    $exponent++;
-	}
-    } else {
-	while ($f != int($f)) {
-	    $f *= 2;
-	    $exponent--;
-	}
-    }
-    my $mantissa = sprintf("%.0f", $f);
-    return ($mantissa, $exponent);
-}
-
-# OP_STRINGIFY is a listop, but it only ever has one arg
-sub pp_stringify {
-    my ($self, $op, $cx) = @_;
-    my $kid = $op->first->sibling;
-    my @other_ops = ();
-    while ($kid->name eq 'null' && !B::Deparse::null($kid->first)) {
-        push(@other_ops, $kid);
-	$kid = $kid->first;
-    }
-    my $info;
-    if ($kid->name =~ /^(?:const|padsv|rv2sv|av2arylen|gvsv|multideref
-			  |aelemfast(?:_lex)?|[ah]elem|join|concat)\z/x) {
-	$info = maybe_targmy(@_, \&dquote);
-    }
-    else {
-	# Actually an optimised join.
-	my $info = listop(@_,"join");
-	$info->{text} =~ s/join([( ])/join$1$self->{'ex_const'}, /;
-    }
-    push @{$info->{other_ops}}, @other_ops;
-    return $info;
-}
-
-
 # Like dq(), but different
 sub re_dq {
     my $self = shift;
@@ -960,12 +719,6 @@ sub regcomp
     return ($self->deparse($kid, $cx, $op), 0, $op);
 }
 
-# osmic acid -- see osmium tetroxide
-
-sub pp_match { matchop(@_, "m", "/") }
-sub pp_pushre { matchop(@_, "m", "/") }
-sub pp_qr { matchop(@_, "qr", "") }
-
 sub pp_split
 {
     my($self, $op, $cx) = @_;
@@ -1048,117 +801,7 @@ sub pp_split
     return info_from_list($op, $self, \@expr_texts, $sep, $type, $opts);
 }
 
-# Kind of silly, but we prefer, subst regexp flags joined together to
-# make words. For example: s/a/b/xo => s/a/b/ox
-
-# oxime -- any of various compounds obtained chiefly by the action of
-# hydroxylamine on aldehydes and ketones and characterized by the
-# bivalent grouping C=NOH [Webster's Tenth]
-
-unless (caller) {
-    eval "use Data::Printer;";
-
-    eval {
-	sub fib($) {
-	    my $x = shift;
-	    return 1 if $x <= 1;
-	    return(fib($x-1) + fib($x-2))
-	}
-	sub baz {
-	    no strict;
-	    CORE::wait;
-	}
-    };
-
-    # use B::Deparse;
-    # my $deparse_old = B::Deparse->new("-l", "-sC");
-    # print $deparse_old->coderef2text(\&baz);
-    # exit 1;
-    my $deparse = __PACKAGE__->new("-l", "-c", "-sC");
-    my $info = $deparse->coderef2info(\&baz);
-    import Data::Printer colored => 0;
-    Data::Printer::p($info);
-    print "\n", '=' x 30, "\n";
-    # print $deparse->($deparse->deparse_subname('fib')->{text});
-    # print "\n", '=' x 30, "\n";
-    # print "\n", '-' x 30, "\n";
-    while (my($key, $value) = each %{$deparse->{optree}}) {
-	my $parent_op_name = 'undef';
-	if ($value->{parent}) {
-	    my $parent = $deparse->{optree}{$value->{parent}};
-	    $parent_op_name = $parent->{op}->name if $parent->{op};
-	}
-	printf("0x%x %s/%s of %s |\n%s",
-	       $key, $value->{op}->name, $value->{type},
-	       $parent_op_name, $deparse->indent($value->{text}));
-	printf " ## line %s\n", $value->{cop} ? $value->{cop}->line : 'undef';
-	print '-' x 30, "\n";
-    }
-}
-
 # Not in Perl 5.20 and presumably < 5.20. No harm in adding to 5.20?
 *pp_ncomplement = *pp_complement;
-sub pp_scomplement { maybe_targmy(@_, \&pfixop, "~.", 21) }
-
-unless (caller) {
-    eval "use Data::Printer;";
-
-    eval {
-	our($Fileparse_fstype);
-	sub fib($) {
-	    my $x = shift;
-	    return 1 if $x <= 1;
-	    return(fib($x-1) + fib($x-2))
-	}
-	sub fileparse {
-	    no strict;
-  # my($fullname,@suffices) = @_;
-
-  my $tail   = '';
-  $tail = $1 . $tail;
-
-  # Ensure taint is propagated from the path to its pieces.
-  $tail .= $taint;
-  wantarray ? ($basename .= $taint, $dirpath .= $taint, $tail)
-            : ($basename .= $taint);
-}
-	sub baz {
-	    no strict;
-	    if ($basename =~ s/$pat//s) {
-	    }
-	}
-    };
-
-    my $deparse = __PACKAGE__->new("-l", "-c");
-    my $info = $deparse->coderef2info(\&fileparse);
-    # my $info = $deparse->coderef2info(\&baz);
-    import Data::Printer colored => 0;
-    Data::Printer::p($info);
-    print "\n", '=' x 30, "\n";
-    # print $deparse->indent($deparse->deparse_subname('fib')->{text});
-    # print "\n", '=' x 30, "\n";
-    # print "\n", '-' x 30, "\n";
-    while (my($key, $value) = each %{$deparse->{optree}}) {
-	my $parent_op_name = 'undef';
-	if ($value->{parent}) {
-	    my $parent = $deparse->{optree}{$value->{parent}};
-	    $parent_op_name = $parent->{op}->name if $parent->{op};
-	}
-	if (eval{$value->{op}->name}) {
-	    printf("0x%x %s/%s of %s |\n%s",
-		   $key, $value->{op}->name, $value->{type},
-		   $parent_op_name, $deparse->{text});
-	} else {
-	    printf("0x%x %s of %s |\n",
-		   $key, $value->{type},
-		   $parent_op_name);
-	}
-	printf " ## line %s\n", $value->{cop} ? $value->{cop}->line : 'undef';
-	print '-' x 30, "\n";
-    }
-    # use B::Deparse;
-    # my $deparse_old = B::Deparse->new("-l", "-sC");
-    # print $deparse_old->coderef2text(\&baz);
-}
 
 1;
