@@ -51,7 +51,7 @@ sub format_info_short($$)
     }
     if (exists $i{maybe_parens}) {
 	my %mp = %{$i{maybe_parens}};
-	if (B::DeparseTree::Node::parens_test($info, $mp{cx}, $mp{prec})) {
+	if (B::DeparseTree::TreeNode::parens_test($info, $mp{cx}, $mp{prec})) {
 	    $text .= ' - parens';
 	}
     }
@@ -115,8 +115,9 @@ EOF
 	    $text .= sprintf "%s: %g\n", $key, $maybe_parens{$key};
 	}
 	$text .= sprintf("need parens: %s\n",
-			 B::DeparseTree::Node::parens_test($info, $maybe_parens{context},
-							     $maybe_parens{precedence}) ?
+			 B::DeparseTree::TreeNode::parens_test($info,
+							       $maybe_parens{context},
+							       $maybe_parens{precedence}) ?
 			 'yes' : 'no');
     }
     return $text;
